@@ -63,9 +63,9 @@ typedef struct mFile {
  * Most data in a GIF image is binary and is treated as such. However, a few
  * key bits are stashed in ASCII. If we try to compare those pieces to the
  * char they represent, it will fail on any non-ASCII (eg, EBCDIC) system. To
- * accomodate these systems, we test against the numeric value of the ASCII
+ * accommodate these systems, we test against the numeric value of the ASCII
  * characters instead of the characters themselves. This is encoding
- * independant.
+ * independent.
  */
 
 static const char GIF87a[] = {			/* ASCII GIF87a */
@@ -624,6 +624,13 @@ FileReadGIF(
 		    imageHeight, colorMap, 0, 0, 0, -1) != TCL_OK) {
 		goto error;
 	    }
+
+	    /*
+	     * This extension starts a new scope, so Graphic control Extension
+	     * data should be cleared
+	     */
+	    transparent = -1;
+
 	    continue;
 	}
 	break;
@@ -943,6 +950,11 @@ DoExtension(
 
     switch (label) {
     case 0x01:			/* Plain Text Extension */
+	/*
+	 * This extension starts a new scope, so Graphic control Extension
+	 * data should be cleared
+	 */
+	*transparent = -1;
 	break;
 
     case 0xff:			/* Application Extension */
@@ -1034,14 +1046,14 @@ ReadImage(
     int transparent)
 {
     unsigned char initialCodeSize;
-    int xpos = 0, ypos = 0, pass = 0, i;
-    register unsigned char *pixelPtr;
+    int xpos = 0, ypos = 0, pass = 0, i, count;
+    unsigned char *pixelPtr;
     static const int interlaceStep[] = { 8, 8, 4, 2 };
     static const int interlaceStart[] = { 0, 4, 2, 1 };
     unsigned short prefix[(1 << MAX_LWZ_BITS)];
     unsigned char append[(1 << MAX_LWZ_BITS)];
     unsigned char stack[(1 << MAX_LWZ_BITS)*2];
-    register unsigned char *top;
+    unsigned char *top;
     int codeSize, clearCode, inCode, endCode, oldCode, maxCode;
     int code, firstCode, v;
 
@@ -1252,6 +1264,25 @@ ReadImage(
 	}
 	pixelPtr = imagePtr + (ypos) * len * ((transparent>=0)?4:3);
     }
+
+    /*
+     * Now read until the final zero byte.
+     * It was observed that there might be 1 length blocks
+     * (test imgPhoto-14.1) which are not read.
+     *
+     * The field "stack" is abused for temporary buffer. it has 4096 bytes
+     * and we need 256.
+     *
+     * Loop until we hit a 0 length block which is the end sign.
+     */
+    while ( 0 < (count = GetDataBlock(gifConfPtr, chan, stack)))
+    {
+	if (-1 == count ) {
+	    Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+		    "error reading GIF image: %s", Tcl_PosixError(interp)));
+	    return TCL_ERROR;
+	}
+    }
     return TCL_OK;
 }
 
@@ -1406,7 +1437,7 @@ Mread(
     size_t numChunks,		/* number of chunks */
     MFile *handle)		/* mmdecode "file" handle */
 {
-    register int i, c;
+    int i, c;
     int count = chunkSize * numChunks;
 
     for (i=0; i<count && (c=Mgetc(handle)) != GIF_DONE; i++) {
@@ -2138,9 +2169,9 @@ ClearHashTable(			/* Reset code table. */
     GIFState_t *statePtr,
     int hSize)
 {
-    register int *hashTablePtr = statePtr->hashTable + hSize;
-    register long i;
-    register long m1 = -1;
+    int *hashTablePtr = statePtr->hashTable + hSize;
+    long i;
+    long m1 = -1;
 
     i = hSize - 16;
     do {			/* might use Sys V memset(3) here */
