@@ -5,7 +5,7 @@
  *	functions.
  *
  * Copyright (c) 1995-1997 Sun Microsystems, Inc.
- * Copyright 2001-2009, Apple Inc.
+ * Copyright (c) 2001-2009, Apple Inc.
  * Copyright (c) 2005-2009 Daniel A. Steffen <das@users.sourceforge.net>
  * Copyright (c) 2017 Marc Culler
  *
@@ -14,9 +14,11 @@
  */
 
 #include "tkMacOSXPrivate.h"
+#include "tkMacOSXConstants.h"
 #include <dlfcn.h>
 #include <objc/objc-auto.h>
 #include <sys/stat.h>
+#include <sys/utsname.h>
 
 static char tkLibPath[PATH_MAX + 1] = "";
 
@@ -41,6 +43,44 @@ static int		TkMacOSXGetAppPathCmd(ClientData cd, Tcl_Interp *ip,
 @synthesize macOSVersion = _macOSVersion;
 @synthesize isDrawing = _isDrawing;
 @synthesize needsToDraw = _needsToDraw;
+@synthesize tkLiveResizeEnded = _tkLiveResizeEnded;
+@synthesize tkPointerWindow = _tkPointerWindow;
+- (void) setTkPointerWindow: (TkWindow *)winPtr
+{
+    if (_tkPointerWindow) {
+	Tcl_Release(_tkPointerWindow);
+    }
+    if (winPtr) {
+	Tcl_Preserve(winPtr);
+    }
+    _tkPointerWindow = winPtr;
+    return;
+}
+@synthesize tkEventTarget = _tkEventTarget;
+- (void) setTkEventTarget: (TkWindow *)winPtr
+{
+    if (_tkEventTarget) {
+	Tcl_Release(_tkEventTarget);
+    }
+    if (winPtr) {
+	Tcl_Preserve(winPtr);
+    }
+    _tkEventTarget = winPtr;
+    return;
+}
+@synthesize tkDragTarget = _tkDragTarget;
+- (void) setTkDragTarget: (TkWindow *)winPtr
+{
+    if (_tkDragTarget) {
+	Tcl_Release(_tkDragTarget);
+    }
+    if (winPtr) {
+	Tcl_Preserve(winPtr);
+    }
+    _tkDragTarget = winPtr;
+    return;
+}
+@synthesize tkButtonState = _tkButtonState;
 @end
 
 /*
@@ -103,18 +143,38 @@ static int		TkMacOSXGetAppPathCmd(ClientData cd, Tcl_Interp *ip,
 #endif
     [self _setupWindowNotifications];
     [self _setupApplicationNotifications];
+
+    if ([NSApp macOSVersion] >= 110000) {
+
+   /*
+    * Initialize Apple Event processing. Apple's docs (see
+    * https://developer.apple.com/documentation/appkit/nsapplication)
+    * recommend doing this here, although historically we have
+    * done this in applicationWillFinishLaunching. In response to
+    * bug 7bb246b072.
+    */
+
+    TkMacOSXInitAppleEvents(_eventInterp);
+
+    }
 }
 
 -(void)applicationDidFinishLaunching:(NSNotification *)notification
 {
     (void)notification;
 
+   if ([NSApp macOSVersion] < 110000) {
+
    /*
-    * Initialize event processing.
+    * Initialize Apple Event processing on macOS versions
+    * older than Big Sur (11).
     */
 
     TkMacOSXInitAppleEvents(_eventInterp);
 
+    }
+
+
     /*
      * Initialize the graphics context.
      */
@@ -140,6 +200,20 @@ static int		TkMacOSXGetAppPathCmd(ClientData cd, Tcl_Interp *ip,
     [NSApp setActivationPolicy:NSApplicationActivationPolicyRegular];
     [NSApp activateIgnoringOtherApps: YES];
 
+    /*
+     * Add an event monitor so we continue to receive NSMouseMoved and
+     * NSMouseDragged events when the mouse moves outside of the key
+     * window. The handler simply returns the events it receives, so
+     * they can be processed in the same way as for other events.
+     */
+
+    [NSEvent addLocalMonitorForEventsMatchingMask:(NSMouseMovedMask |
+						   NSLeftMouseDraggedMask)
+	 handler:^NSEvent *(NSEvent *event)
+	 {
+	     return event;
+	 }];
+
     /*
      * Process events to ensure that the root window is fully initialized. See
      * ticket 56a1823c73.
@@ -168,6 +242,7 @@ static int		TkMacOSXGetAppPathCmd(ClientData cd, Tcl_Interp *ip,
      */
 
     int minorVersion, majorVersion;
+
 #if MAC_OS_X_VERSION_MAX_ALLOWED < 101000
     Gestalt(gestaltSystemVersionMinor, (SInt32*)&minorVersion);
     majorVersion = 10;
@@ -177,6 +252,24 @@ static int		TkMacOSXGetAppPathCmd(ClientData cd, Tcl_Interp *ip,
     majorVersion = systemVersion.majorVersion;
     minorVersion = systemVersion.minorVersion;
 #endif
+
+    if (majorVersion == 10 && minorVersion == 16) {
+
+	/*
+	 * If a program compiled with a macOS 10.XX SDK is run on macOS 11.0 or
+	 * later then it will report majorVersion 10 and minorVersion 16, no
+	 * matter what the actual OS version of the host may be. And of course
+	 * Apple never released macOS 10.16. To work around this we guess the
+	 * OS version from the kernel release number, as reported by uname.
+	 */
+
+	struct utsname name;
+	char *endptr;
+	if (uname(&name) == 0) {
+	    majorVersion = strtol(name.release, &endptr, 10) - 9;
+	    minorVersion = 0;
+	}
+    }
     [NSApp setMacOSVersion: 10000*majorVersion + 100*minorVersion];
 
     /*
@@ -203,6 +296,7 @@ static int		TkMacOSXGetAppPathCmd(ClientData cd, Tcl_Interp *ip,
 	if (path) {
 	    NSImage *image = [[NSImage alloc] initWithContentsOfFile:path];
 	    if (image) {
+		[image setName:@"NSApplicationIcon"];
 		[NSApp setApplicationIconImage:image];
 		[image release];
 	    }
@@ -354,7 +448,7 @@ TkpInit(
 
     /*
      * TkpInit can be called multiple times with different interpreters. But
-     * The application initialization should only be done onece.
+     * The application initialization should only be done once.
      */
 
     if (!initialized) {
@@ -415,7 +509,7 @@ TkpInit(
          * the application icon, will be delivered before the procedure meant
          * to to handle the AppleEvent has been defined.  This is handled in
          * tkMacOSXHLEvents.c by scheduling a timer event to handle the
-         * ApplEvent later, after the required procedure has been defined.
+         * AppleEvent later, after the required procedure has been defined.
          */
 
 	[NSApp _setup:interp];
