@@ -105,7 +105,7 @@ XDestroyWindow(
     macWin->view = nil;
 
     /*
-     * Delay deletion of a toplevel data structure untill all children have
+     * Delay deletion of a toplevel data structure until all children have
      * been deleted.
      */
 
@@ -148,6 +148,8 @@ XMapWindow(
     TkWindow *winPtr = macWin->winPtr;
     NSWindow *win = TkMacOSXGetNSWindowForDrawable(window);
     static Bool initialized = NO;
+    NSPoint mouse = [NSEvent mouseLocation];
+    int x = mouse.x, y = TkMacOSXZeroScreenHeight() - mouse.y;
 
     /*
      * Under certain situations it's possible for this function to be called
@@ -185,6 +187,16 @@ XMapWindow(
 		    [win orderFrontRegardless];
 		}
 	    }
+
+	    /*
+	     * Call Tk_UpdatePointer to tell Tk whether the pointer is in the
+	     * new window.
+	     */
+
+	    NSPoint viewLocation = [view convertPoint:mouse fromView:nil];
+	    if (NSPointInRect(viewLocation, NSInsetRect([view bounds], 2, 2))) {
+		Tk_UpdatePointer((Tk_Window) winPtr, x, y, [NSApp tkButtonState]);
+	    }
 	} else {
 	    TkWindow *contWinPtr = TkpGetOtherWindow(winPtr);
 
@@ -303,12 +315,41 @@ XUnmapWindow(
     display->request++;
     if (Tk_IsTopLevel(winPtr)) {
 	if (!Tk_IsEmbedded(winPtr) &&
-		winPtr->wmInfoPtr->hints.initial_state!=IconicState) {
-	    [win orderOut:nil];
+	    winPtr->wmInfoPtr->hints.initial_state!=IconicState) {
 	    [win setExcludedFromWindowsMenu:YES];
+	    [win orderOut:NSApp];
+	    if ([win isKeyWindow]) {
+
+		/*
+		 * If we are unmapping the key window then we need to make sure
+		 * that a new key window is assigned, if possible.  This is
+		 * supposed to happen when a key window is ordered out, but as
+		 * noted in tkMacOSXWm.c this does not happen, in spite of
+		 * Apple's claims to the contrary.
+		 */
+
+		for (NSWindow *w in [NSApp orderedWindows]) {
+		    TkWindow *winPtr2 = TkMacOSXGetTkWindow(w);
+		    WmInfo *wmInfoPtr;
+
+		    BOOL isOnScreen;
+
+		    if (!winPtr2 || !winPtr2->wmInfoPtr) {
+			continue;
+		    }
+		    wmInfoPtr = winPtr2->wmInfoPtr;
+		    isOnScreen = (wmInfoPtr->hints.initial_state != IconicState &&
+				  wmInfoPtr->hints.initial_state != WithdrawnState);
+		    if (w != win && isOnScreen && [w canBecomeKeyWindow]) {
+			[w makeKeyAndOrderFront:NSApp];
+			break;
+		    }
+		}
+	    }
 	}
 	TkMacOSXInvalClipRgns((Tk_Window)winPtr);
     } else {
+
 	/*
 	 * Rebuild the visRgn clip region for the parent so it will be allowed
 	 * to draw in the space from which this subwindow was removed and then
@@ -982,7 +1023,7 @@ InvalViewRect(
     void *ref)
 {
     static CGAffineTransform t;
-    TKContentView *view = ref;
+    TKContentView *view = (TKContentView *)ref;
     NSRect dirtyRect;
 
     if (!view) {
@@ -1467,7 +1508,7 @@ Tk_FreePixmap(
 
     display->request++;
     if (macPix->context) {
-	char *data = CGBitmapContextGetData(macPix->context);
+	char *data = (char *)CGBitmapContextGetData(macPix->context);
 
 	if (data) {
 	    ckfree(data);
