@@ -33,9 +33,6 @@
 
 #define ROOT_ID 10
 
-CGFloat tkMacOSXZeroScreenHeight = 0;
-CGFloat tkMacOSXZeroScreenTop = 0;
-
 /*
  * Declarations of static variables used in this file.
  */
@@ -90,13 +87,9 @@ TkMacOSXDisplayChanged(
     nsScreens = [NSScreen screens];
     if (nsScreens && [nsScreens count]) {
 	NSScreen *s = [nsScreens objectAtIndex:0];
-	NSRect bounds = [s frame], visible = [s visibleFrame];
+	NSRect bounds = [s frame];
 	NSRect maxBounds = NSZeroRect;
 
-	tkMacOSXZeroScreenHeight = bounds.size.height;
-	tkMacOSXZeroScreenTop = tkMacOSXZeroScreenHeight -
-		(visible.origin.y + visible.size.height);
-
 	screen->root_depth = NSBitsPerPixelFromDepth([s depth]);
 	screen->width = bounds.size.width;
 	screen->height = bounds.size.height;
@@ -110,6 +103,67 @@ TkMacOSXDisplayChanged(
     }
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkMacOSXZeroScreenHeight --
+ *
+ *	Replacement for the tkMacOSXZeroScreenHeight variable to avoid
+ *	caching values from NSScreen (fixes bug aea00be199).
+ *
+ * Results:
+ *	Returns the height of screen 0 (the screen assigned the menu bar
+ *	in System Preferences), or 0.0 if getting [NSScreen screens] fails.
+ *
+ * Side effects:
+ *	None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+CGFloat
+TkMacOSXZeroScreenHeight()
+{
+    NSArray *nsScreens = [NSScreen screens];
+    if (nsScreens && [nsScreens count]) {
+	NSScreen *s = [nsScreens objectAtIndex:0];
+	NSRect bounds = [s frame];
+	return bounds.size.height;
+    }
+    return 0.0;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkMacOSXZeroScreenTop --
+ *
+ *	Replacement for the tkMacOSXZeroScreenTop variable to avoid
+ *	caching values from visibleFrame.
+ *
+ * Results:
+ *	Returns how far below the top of screen 0 to draw
+ *	(i.e. the height of the menu bar if it is always shown),
+ *	or 0.0 if getting [NSScreen screens] fails.
+ *
+ * Side effects:
+ *	None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+CGFloat
+TkMacOSXZeroScreenTop()
+{
+    NSArray *nsScreens = [NSScreen screens];
+    if (nsScreens && [nsScreens count]) {
+	NSScreen *s = [nsScreens objectAtIndex:0];
+	NSRect bounds = [s frame], visible = [s visibleFrame];
+	return bounds.size.height - (visible.origin.y + visible.size.height);
+    }
+    return 0.0;
+}
+
 /*
  *----------------------------------------------------------------------
  *
@@ -147,8 +201,8 @@ TkpOpenDisplay(
 	}
     }
 
-    display = ckalloc(sizeof(Display));
-    screen  = ckalloc(sizeof(Screen));
+    display = (Display *)ckalloc(sizeof(Display));
+    screen = (Screen *)ckalloc(sizeof(Screen));
     bzero(display, sizeof(Display));
     bzero(screen, sizeof(Screen));
 
@@ -178,7 +232,7 @@ TkpOpenDisplay(
     {
 	int major, minor, patch;
 
-#if MAC_OS_X_VERSION_MIN_REQUIRED < 101000
+#if MAC_OS_X_VERSION_MAX_ALLOWED < 101000
 	Gestalt(gestaltSystemVersionMajor, (SInt32*)&major);
 	Gestalt(gestaltSystemVersionMinor, (SInt32*)&minor);
 	Gestalt(gestaltSystemVersionBugFix, (SInt32*)&patch);
@@ -196,13 +250,13 @@ TkpOpenDisplay(
      */
     screen->root	= ROOT_ID;
     screen->display	= display;
-    screen->black_pixel = 0x00000000 | PIXEL_MAGIC << 24;
-    screen->white_pixel = 0x00FFFFFF | PIXEL_MAGIC << 24;
+    screen->black_pixel = 0x00000000;
+    screen->white_pixel = 0x00FFFFFF;
     screen->ext_data	= (XExtData *) &maxBounds;
 
-    screen->root_visual = ckalloc(sizeof(Visual));
+    screen->root_visual = (Visual *)ckalloc(sizeof(Visual));
     screen->root_visual->visualid     = 0;
-    screen->root_visual->class	      = TrueColor;
+    screen->root_visual->c_class      = TrueColor;
     screen->root_visual->red_mask     = 0x00FF0000;
     screen->root_visual->green_mask   = 0x0000FF00;
     screen->root_visual->blue_mask    = 0x000000FF;
@@ -215,7 +269,7 @@ TkpOpenDisplay(
 
     TkMacOSXDisplayChanged(display);
 
-    gMacDisplay = ckalloc(sizeof(TkDisplay));
+    gMacDisplay = (TkDisplay *)ckalloc(sizeof(TkDisplay));
 
     /*
      * This is the quickest way to make sure that all the *Init flags get
@@ -225,6 +279,12 @@ TkpOpenDisplay(
     bzero(gMacDisplay, sizeof(TkDisplay));
     gMacDisplay->display = display;
     [pool drain];
+
+    /*
+     * Key map info must be available immediately, because of "send event".
+     */
+    TkpInitKeymapInfo(gMacDisplay);
+
     return gMacDisplay;
 }
 
@@ -327,7 +387,7 @@ TkClipCleanup(
 
 static XID
 MacXIdAlloc(
-    Display *display)		/* Display for which to allocate. */
+    TCL_UNUSED(Display *))		/* Display for which to allocate. */
 {
     static long int cur_id = 100;
     /*
@@ -357,8 +417,8 @@ MacXIdAlloc(
 
 static int
 DefaultErrorHandler(
-    Display* display,
-    XErrorEvent* err_evt)
+    TCL_UNUSED(Display *),
+    TCL_UNUSED(XErrorEvent *))
 {
     /*
      * This call should never be called. Tk replaces it with its own error
@@ -371,8 +431,8 @@ DefaultErrorHandler(
 
 char *
 XGetAtomName(
-    Display * display,
-    Atom atom)
+    Display *display,
+    TCL_UNUSED(Atom))
 {
     display->request++;
     return NULL;
@@ -380,7 +440,7 @@ XGetAtomName(
 
 XErrorHandler
 XSetErrorHandler(
-    XErrorHandler handler)
+    TCL_UNUSED(XErrorHandler))
 {
     return DefaultErrorHandler;
 }
@@ -388,7 +448,7 @@ XSetErrorHandler(
 Window
 XRootWindow(
     Display *display,
-    int screen_number)
+    TCL_UNUSED(int))
 {
     display->request++;
     return ROOT_ID;
@@ -406,7 +466,7 @@ XGetGeometry(
     unsigned int *border_width_return,
     unsigned int *depth_return)
 {
-    TkWindow *winPtr = ((MacDrawable *) d)->winPtr;
+    TkWindow *winPtr = ((MacDrawable *)d)->winPtr;
 
     display->request++;
     *root_return = ROOT_ID;
@@ -418,7 +478,7 @@ XGetGeometry(
 	*border_width_return = winPtr->changes.border_width;
 	*depth_return = Tk_Depth(winPtr);
     } else {
-	CGSize size = ((MacDrawable *) d)->size;
+	CGSize size = ((MacDrawable *)d)->size;
 	*x_return = 0;
 	*y_return =  0;
 	*width_return = size.width;
@@ -429,65 +489,43 @@ XGetGeometry(
     return 1;
 }
 
-void
+int
 XChangeProperty(
-    Display* display,
-    Window w,
-    Atom property,
-    Atom type,
-    int format,
-    int mode,
-    _Xconst unsigned char* data,
-    int nelements)
+    TCL_UNUSED(Display *),
+    TCL_UNUSED(Window),
+    TCL_UNUSED(Atom),
+    TCL_UNUSED(Atom),
+    TCL_UNUSED(int),
+    TCL_UNUSED(int),
+    TCL_UNUSED(_Xconst unsigned char *),
+    TCL_UNUSED(int))
 {
     Debugger();
+    return Success;
 }
 
-void
+int
 XSelectInput(
-    Display* display,
-    Window w,
-    long event_mask)
+    TCL_UNUSED(Display *),
+    TCL_UNUSED(Window),
+    TCL_UNUSED(long))
 {
     Debugger();
+    return Success;
 }
 
 int
 XBell(
-    Display* display,
-    int percent)
+    TCL_UNUSED(Display *),
+    TCL_UNUSED(int))
 {
     NSBeep();
     return Success;
 }
 
-#if 0
-void
-XSetWMNormalHints(
-    Display* display,
-    Window w,
-    XSizeHints* hints)
-{
-    /*
-     * Do nothing. Shouldn't even be called.
-     */
-}
-
-XSizeHints *
-XAllocSizeHints(void)
-{
-    /*
-     * Always return NULL. Tk code checks to see if NULL is returned & does
-     * nothing if it is.
-     */
-
-    return NULL;
-}
-#endif
-
 GContext
 XGContextFromGC(
-    GC gc)
+    TCL_UNUSED(GC))
 {
     /*
      * TODO: currently a no-op
@@ -498,65 +536,43 @@ XGContextFromGC(
 
 Status
 XSendEvent(
-    Display* display,
-    Window w,
-    Bool propagate,
-    long event_mask,
-    XEvent* event_send)
+    TCL_UNUSED(Display *),
+    TCL_UNUSED(Window),
+    TCL_UNUSED(Bool),
+    TCL_UNUSED(long),
+    TCL_UNUSED(XEvent *))
 {
     Debugger();
     return 0;
 }
 
-void
+int
 XClearWindow(
-    Display* display,
-    Window w)
-{
-}
-
-/*
-int
-XDrawPoint(
-    Display* display,
-    Drawable d,
-    GC gc,
-    int x,
-    int y)
-{
-}
-
-int
-XDrawPoints(
-    Display* display,
-    Drawable d,
-    GC gc,
-    XPoint* points,
-    int npoints,
-    int mode)
-{
-}
-*/
-
-int
-XWarpPointer(
-    Display* display,
-    Window src_w,
-    Window dest_w,
-    int src_x,
-    int src_y,
-    unsigned int src_width,
-    unsigned int src_height,
-    int dest_x,
-    int dest_y)
+    TCL_UNUSED(Display *),
+    TCL_UNUSED(Window))
 {
     return Success;
 }
 
-void
+int
+XWarpPointer(
+    TCL_UNUSED(Display *),
+    TCL_UNUSED(Window),
+    TCL_UNUSED(Window),
+    TCL_UNUSED(int),
+    TCL_UNUSED(int),
+    TCL_UNUSED(unsigned int),
+    TCL_UNUSED(unsigned int),
+    TCL_UNUSED(int),
+    TCL_UNUSED(int))
+{
+    return Success;
+}
+
+int
 XQueryColor(
-    Display* display,
-    Colormap colormap,
+    TCL_UNUSED(Display *),
+    TCL_UNUSED(Colormap),
     XColor* def_in_out)
 {
     unsigned long p;
@@ -572,12 +588,13 @@ XQueryColor(
     d->blue	= (b << 8) | b;
     d->flags	= DoRed|DoGreen|DoBlue;
     d->pad	= 0;
+    return Success;
 }
 
-void
+int
 XQueryColors(
-    Display* display,
-    Colormap colormap,
+    TCL_UNUSED(Display *),
+    TCL_UNUSED(Colormap),
     XColor* defs_in_out,
     int ncolors)
 {
@@ -597,17 +614,17 @@ XQueryColors(
 	d->flags	= DoRed|DoGreen|DoBlue;
 	d->pad		= 0;
     }
+    return Success;
 }
 
 int
-XQueryTree(display, w, root_return, parent_return, children_return,
-	nchildren_return)
-    Display* display;
-    Window w;
-    Window* root_return;
-    Window* parent_return;
-    Window** children_return;
-    unsigned int* nchildren_return;
+XQueryTree(
+    TCL_UNUSED(Display *),
+    TCL_UNUSED(Window),
+    TCL_UNUSED(Window *),
+    TCL_UNUSED(Window *),
+    TCL_UNUSED(Window **),
+    TCL_UNUSED(unsigned int *))
 {
     return 0;
 }
@@ -616,17 +633,17 @@ XQueryTree(display, w, root_return, parent_return, children_return,
 int
 XGetWindowProperty(
     Display *display,
-    Window w,
-    Atom property,
-    long long_offset,
-    long long_length,
-    Bool delete,
-    Atom req_type,
+    TCL_UNUSED(Window),
+    TCL_UNUSED(Atom),
+    TCL_UNUSED(long),
+    TCL_UNUSED(long),
+    TCL_UNUSED(Bool),
+    TCL_UNUSED(Atom),
     Atom *actual_type_return,
     int *actual_format_return,
     unsigned long *nitems_return,
     unsigned long *bytes_after_return,
-    unsigned char ** prop_return)
+    TCL_UNUSED(unsigned char **))
 {
     display->request++;
     *actual_type_return = None;
@@ -635,30 +652,32 @@ XGetWindowProperty(
     return 0;
 }
 
-void
+int
 XRefreshKeyboardMapping(
-    XMappingEvent *x)
+    TCL_UNUSED(XMappingEvent *))
 {
     /* used by tkXEvent.c */
     Debugger();
+    return Success;
 }
 
-void
+int
 XSetIconName(
     Display* display,
-    Window w,
-    const char *icon_name)
+    TCL_UNUSED(Window),
+    TCL_UNUSED(const char *))
 {
     /*
      * This is a no-op, no icon name for Macs.
      */
     display->request++;
+    return Success;
 }
 
-void
+int
 XForceScreenSaver(
     Display* display,
-    int mode)
+    TCL_UNUSED(int))
 {
     /*
      * This function is just a no-op. It is defined to reset the screen saver.
@@ -667,12 +686,13 @@ XForceScreenSaver(
      */
 
     display->request++;
+    return Success;
 }
 
 void
 Tk_FreeXId(
-    Display *display,
-    XID xid)
+    TCL_UNUSED(Display *),
+    TCL_UNUSED(XID))
 {
     /* no-op function needed for stubs implementation. */
 }
@@ -680,14 +700,26 @@ Tk_FreeXId(
 int
 XSync(
     Display *display,
-    Bool flag)
+    TCL_UNUSED(Bool))
 {
-    TkMacOSXFlushWindows();
+    /*
+     *  The main use of XSync is by the update command, which alternates
+     *  between running an event loop to process all events without waiting and
+     *  calling XSync on all displays until no events are left.  On X11 the
+     *  call to XSync might cause the window manager to generate more events
+     *  which would then get processed. Apparently this process stabilizes on
+     *  X11, leaving the window manager in a state where all events have been
+     *  generated and no additional events can be genereated by updating widgets.
+     *
+     *  It is not clear what the Aqua port should do when XSync is called, but
+     *  currently the best option seems to be to do nothing.  (See ticket
+     *  [da5f2266df].)
+     */
+
     display->request++;
     return 0;
 }
 
-#if 0
 int
 XSetClipRectangles(
     Display *d,
@@ -696,24 +728,22 @@ XSetClipRectangles(
     int clip_y_origin,
     XRectangle* rectangles,
     int n,
-    int ordering)
+    TCL_UNUSED(int))
 {
     TkRegion clipRgn = TkCreateRegion();
 
     while (n--) {
-	XRectangle rect = *rectangles;
+    	XRectangle rect = *rectangles;
 
-	rect.x += clip_x_origin;
-	rect.y += clip_y_origin;
-	TkUnionRectWithRegion(&rect, clipRgn, clipRgn);
-	rectangles++;
+    	rect.x += clip_x_origin;
+    	rect.y += clip_y_origin;
+    	TkUnionRectWithRegion(&rect, clipRgn, clipRgn);
+    	rectangles++;
     }
     TkSetRegion(d, gc, clipRgn);
     TkDestroyRegion(clipRgn);
     return 1;
 }
-#endif
-
 /*
  *----------------------------------------------------------------------
  *
@@ -770,90 +800,158 @@ TkGetServerInfo(
  *----------------------------------------------------------------------
  */
 
-void
+int
 XChangeWindowAttributes(
-    Display *display,
-    Window w,
-    unsigned long value_mask,
-    XSetWindowAttributes *attributes)
+    TCL_UNUSED(Display *),
+    TCL_UNUSED(Window),
+    TCL_UNUSED(unsigned long),
+    TCL_UNUSED(XSetWindowAttributes *))
 {
+    return Success;
 }
 
-void
+int
 XSetWindowBackground(
-    Display *display,
-    Window window,
-    unsigned long value)
+    TCL_UNUSED(Display *),
+    TCL_UNUSED(Window),
+    TCL_UNUSED(unsigned long))
 {
+    return Success;
 }
 
-void
+int
 XSetWindowBackgroundPixmap(
-    Display *display,
-    Window w,
-    Pixmap background_pixmap)
+    TCL_UNUSED(Display *),
+    TCL_UNUSED(Window),
+    TCL_UNUSED(Pixmap))
 {
+    return Success;
 }
 
-void
+int
 XSetWindowBorder(
-    Display *display,
-    Window w,
-    unsigned long border_pixel)
+    TCL_UNUSED(Display *),
+    TCL_UNUSED(Window),
+    TCL_UNUSED(unsigned long))
 {
+    return Success;
 }
 
-void
+int
 XSetWindowBorderPixmap(
-    Display *display,
-    Window w,
-    Pixmap border_pixmap)
+    TCL_UNUSED(Display *),
+    TCL_UNUSED(Window),
+    TCL_UNUSED(Pixmap))
 {
+    return Success;
 }
 
-void
+int
 XSetWindowBorderWidth(
-    Display *display,
-    Window w,
-    unsigned int width)
+    TCL_UNUSED(Display *),
+    TCL_UNUSED(Window),
+    TCL_UNUSED(unsigned int))
 {
+    return Success;
 }
 
-void
+int
 XSetWindowColormap(
-    Display *display,
-    Window w,
-    Colormap colormap)
+    TCL_UNUSED(Display *),
+    TCL_UNUSED(Window),
+    TCL_UNUSED(Colormap))
 {
     Debugger();
+    return Success;
 }
 
 Status
 XStringListToTextProperty(
-    char **list,
-    int count,
-    XTextProperty *text_prop_return)
+    TCL_UNUSED(char **),
+    TCL_UNUSED(int),
+    TCL_UNUSED(XTextProperty *))
 {
     Debugger();
-    return (Status) 0;
+    return Success;
 }
 
 void
 XSetWMClientMachine(
-    Display *display,
-    Window w,
-    XTextProperty *text_prop)
+    TCL_UNUSED(Display *),
+    TCL_UNUSED(Window),
+    TCL_UNUSED(XTextProperty *))
 {
     Debugger();
 }
 
 XIC
-XCreateIC(void)
+XCreateIC(TCL_UNUSED(XIM), ...)
 {
     Debugger();
     return (XIC) 0;
 }
 
+#undef XVisualIDFromVisual
+VisualID
+XVisualIDFromVisual(
+    Visual *visual)
+{
+    return visual->visualid;
+}
+
+#undef XSynchronize
+XAfterFunction
+XSynchronize(
+    Display *display,
+    TCL_UNUSED(Bool))
+{
+    display->request++;
+    return NULL;
+}
+
+#undef XUngrabServer
+int
+XUngrabServer(
+    TCL_UNUSED(Display *))
+{
+    return 0;
+}
+
+#undef XNoOp
+int
+XNoOp(
+    Display *display)
+{
+	display->request++;
+    return 0;
+}
+
+#undef XGrabServer
+int
+XGrabServer(
+    TCL_UNUSED(Display *))
+{
+    return 0;
+}
+
+#undef XFree
+int
+XFree(
+    void *data)
+{
+	if ((data) != NULL) {
+		ckfree(data);
+	}
+    return 0;
+}
+#undef XFlush
+int
+XFlush(
+    TCL_UNUSED(Display *))
+{
+    return 0;
+}
+
 /*
  *----------------------------------------------------------------------
  *
@@ -873,7 +971,7 @@ XCreateIC(void)
 
 const char *
 TkGetDefaultScreenName(
-    Tcl_Interp *interp,		/* Not used. */
+    TCL_UNUSED(Tcl_Interp *),
     const char *screenName)		/* If NULL, use default string. */
 {
     if ((screenName == NULL) || (screenName[0] == '\0')) {
@@ -900,7 +998,7 @@ TkGetDefaultScreenName(
 
 long
 Tk_GetUserInactiveTime(
-    Display *dpy)
+    TCL_UNUSED(Display *))
 {
     io_registry_entry_t regEntry;
     CFMutableDictionaryRef props = NULL;
@@ -909,7 +1007,7 @@ Tk_GetUserInactiveTime(
     uint64_t time;
     IOReturn result;
 
-    regEntry = IOServiceGetMatchingService(kIOMasterPortDefault,
+    regEntry = IOServiceGetMatchingService(0,
 	    IOServiceMatching("IOHIDSystem"));
 
     if (regEntry == 0) {
@@ -966,7 +1064,7 @@ Tk_GetUserInactiveTime(
 
 void
 Tk_ResetUserInactiveTime(
-    Display *dpy)
+    TCL_UNUSED(Display *))
 {
     lastInactivityReset = TkpGetMS();
 }
