@@ -35,9 +35,11 @@ typedef struct UnixButton {
  * The class function table for the button widgets.
  */
 
-Tk_ClassProcs tkpButtonProcs = {
+const Tk_ClassProcs tkpButtonProcs = {
     sizeof(Tk_ClassProcs),	/* size */
     TkButtonWorldChanged,	/* worldChangedProc */
+    NULL,					/* createProc */
+    NULL					/* modalProc */
 };
 
 /*
@@ -55,7 +57,7 @@ Tk_ClassProcs tkpButtonProcs = {
  */
 
 /* XPM */
-static char *button_images[] = {
+static const char *const button_images[] = {
     /* width height ncolors chars_per_pixel */
     "52 26 7 1",
     /* colors */
@@ -171,7 +173,7 @@ TkpDrawCheckIndicator(
      * Sanity check.
      */
 
-    if (tkwin == NULL || display == None || d == None || bgBorder == NULL
+    if (tkwin == NULL || display == NULL || d == None || bgBorder == NULL
 	    || indicatorColor == NULL) {
 	return;
     }
@@ -279,7 +281,7 @@ TkpDrawCheckIndicator(
     for (iy=0 ; iy<dim ; iy++) {
 	for (ix=0 ; ix<dim ; ix++) {
 	    XPutPixel(img, ix, iy,
-		    imgColors[button_images[imgstart+iy][imgsel+ix] - 'A'] );
+		    imgColors[button_images[imgstart+iy][imgsel+ix] - 'A']);
 	}
     }
 
@@ -293,9 +295,9 @@ TkpDrawCheckIndicator(
     copyGC = Tk_GetGC(tkwin, 0, &gcValues);
 
     XPutImage(display, pixmap, copyGC, img, 0, 0, 0, 0,
-	    (unsigned int)dim, (unsigned int)dim);
+	    (unsigned)dim, (unsigned)dim);
     XCopyArea(display, pixmap, d, copyGC, 0, 0,
-	    (unsigned int)dim, (unsigned int)dim, x, y);
+	    (unsigned)dim, (unsigned)dim, x, y);
 
     /*
      * Tidy up.
@@ -324,10 +326,9 @@ TkpDrawCheckIndicator(
 
 TkButton *
 TkpCreateButton(
-    Tk_Window tkwin)
+    TCL_UNUSED(Tk_Window))
 {
-    UnixButton *butPtr = (UnixButton *) ckalloc(sizeof(UnixButton));
-    return (TkButton *) butPtr;
+    return (TkButton *)ckalloc(sizeof(UnixButton));
 }
 
 /*
@@ -348,11 +349,52 @@ TkpCreateButton(
  *----------------------------------------------------------------------
  */
 
+static void
+ShiftByOffset(
+    TkButton *butPtr,
+    int relief,
+    int *x,		/* shift this x coordinate */
+    int *y,		/* shift this y coordinate */
+    int width,		/* width of image/text */
+    int height)		/* height of image/text */
+{
+    if (relief != TK_RELIEF_RAISED
+	    && butPtr->type == TYPE_BUTTON
+	    && !Tk_StrictMotif(butPtr->tkwin)) {
+	int shiftX;
+	int shiftY;
+
+	/*
+	 * This is an (unraised) button widget, so we offset the text to make
+	 * the button appear to move up and down as the relief changes.
+	 */
+
+	shiftX = shiftY = (relief == TK_RELIEF_SUNKEN) ? 2 : 1;
+
+	if (relief != TK_RELIEF_RIDGE) {
+	    /*
+	     * Take back one pixel if the padding is even, otherwise the
+	     * content will be displayed too far right/down.
+	     */
+
+	    if ((Tk_Width(butPtr->tkwin) - width) % 2 == 0) {
+		shiftX -= 1;
+	    }
+	    if ((Tk_Height(butPtr->tkwin) - height) % 2 == 0) {
+		shiftY -= 1;
+	    }
+	}
+
+	*x += shiftX;
+	*y += shiftY;
+    }
+}
+
 void
 TkpDisplayButton(
     ClientData clientData)	/* Information about widget. */
 {
-    register TkButton *butPtr = (TkButton *) clientData;
+    TkButton *butPtr = (TkButton *)clientData;
     GC gc;
     Tk_3DBorder border;
     Pixmap pixmap;
@@ -363,10 +405,6 @@ TkpDisplayButton(
     int width = 0, height = 0, fullWidth, fullHeight;
     int textXOffset, textYOffset;
     int haveImage = 0, haveText = 0;
-    int offset;			/* 1 means this is a button widget, so we
-				 * offset the text to make the button appear
-				 * to move up and down as the relief
-				 * changes. */
     int imageWidth, imageHeight;
     int imageXOffset = 0, imageYOffset = 0;
 				/* image information that will be used to
@@ -429,8 +467,6 @@ TkpDisplayButton(
 	}
     }
 
-    offset = (butPtr->type == TYPE_BUTTON) && !Tk_StrictMotif(butPtr->tkwin);
-
     /*
      * In order to avoid screen flashes, this function redraws the button in a
      * pixmap, then copies the pixmap to the screen in a single operation.
@@ -522,17 +558,7 @@ TkpDisplayButton(
 		butPtr->indicatorSpace + fullWidth, fullHeight, &x, &y);
 
 	x += butPtr->indicatorSpace;
-
-	x += offset;
-	y += offset;
-	if (relief == TK_RELIEF_RAISED) {
-	    x -= offset;
-	    y -= offset;
-	} else if (relief == TK_RELIEF_SUNKEN) {
-	    x += offset;
-	    y += offset;
-	}
-
+	ShiftByOffset(butPtr, relief, &x, &y, width, height);
 	imageXOffset += x;
 	imageYOffset += y;
 
@@ -590,16 +616,7 @@ TkpDisplayButton(
 	    TkComputeAnchor(butPtr->anchor, tkwin, 0, 0,
 		    butPtr->indicatorSpace + width, height, &x, &y);
 	    x += butPtr->indicatorSpace;
-
-	    x += offset;
-	    y += offset;
-	    if (relief == TK_RELIEF_RAISED) {
-		x -= offset;
-		y -= offset;
-	    } else if (relief == TK_RELIEF_SUNKEN) {
-		x += offset;
-		y += offset;
-	    }
+	    ShiftByOffset(butPtr, relief, &x, &y, width, height);
 	    imageXOffset += x;
 	    imageYOffset += y;
 	    if (butPtr->image != NULL) {
@@ -652,16 +669,7 @@ TkpDisplayButton(
 		    butPtr->textHeight, &x, &y);
 
 	    x += butPtr->indicatorSpace;
-
-	    x += offset;
-	    y += offset;
-	    if (relief == TK_RELIEF_RAISED) {
-		x -= offset;
-		y -= offset;
-	    } else if (relief == TK_RELIEF_SUNKEN) {
-		x += offset;
-		y += offset;
-	    }
+	    ShiftByOffset(butPtr, relief, &x, &y, width, height);
 	    Tk_DrawTextLayout(butPtr->display, pixmap, gc, butPtr->textLayout,
 		    x, y, 0, -1);
 	    Tk_UnderlineTextLayout(butPtr->display, pixmap, gc,
@@ -797,8 +805,6 @@ TkpDisplayButton(
 		butPtr->borderWidth, relief);
     }
     if (butPtr->highlightWidth > 0) {
-	GC gc;
-
 	if (butPtr->flags & GOT_FOCUS) {
 	    gc = Tk_GCForColor(butPtr->highlightColorPtr, pixmap);
 	} else {
@@ -850,7 +856,7 @@ TkpDisplayButton(
 
 void
 TkpComputeButtonGeometry(
-    register TkButton *butPtr)	/* Button whose geometry may have changed. */
+    TkButton *butPtr)	/* Button whose geometry may have changed. */
 {
     int width, height, avgWidth, txtWidth, txtHeight;
     int haveImage = 0, haveText = 0;
