@@ -17,37 +17,20 @@
 
 #define __UNIX__ 1
 
-/*
- * Macro to use instead of "void" for arguments that must have
- * type "void *" in ANSI C;  maps them to type "char *" in
- * non-ANSI systems.  This macro may be used in some of the include
- * files below, which is why it is defined here.
- */
-
-#ifndef VOID
-#   ifdef __STDC__
-#       define VOID void
-#   else
-#       define VOID char
-#   endif
-#endif
-
 #include <stdio.h>
-#include <ctype.h>
-#include <fcntl.h>
-#ifndef NO_LIMITS_H
-#   include <limits.h>
-#else
-#   include "../compat/limits.h"
-#endif
-#include <math.h>
 #include <pwd.h>
+#include <assert.h>
+#include <errno.h>
+#include <fcntl.h>
+#include <ctype.h>
+#include <math.h>
+#include <string.h>
+#include <limits.h>
 #ifdef NO_STDLIB_H
 #   include "../compat/stdlib.h"
 #else
 #   include <stdlib.h>
 #endif
-#include <string.h>
 #include <sys/types.h>
 #include <sys/file.h>
 #ifdef HAVE_SYS_SELECT_H
@@ -57,21 +40,27 @@
 #ifndef _TCL
 #   include <tcl.h>
 #endif
-#if TIME_WITH_SYS_TIME
+#ifdef TIME_WITH_SYS_TIME
 #   include <sys/time.h>
 #   include <time.h>
 #else
-#   if HAVE_SYS_TIME_H
-#       include <sys/time.h>
+#   ifdef HAVE_SYS_TIME_H
+#	include <sys/time.h>
 #   else
-#       include <time.h>
+#	include <time.h>
 #   endif
 #endif
+#ifdef HAVE_INTTYPES_H
+#    include <inttypes.h>
+#endif
 #ifndef NO_UNISTD_H
 #   include <unistd.h>
 #else
 #   include "../compat/unistd.h"
 #endif
+#if defined(__GNUC__) && !defined(__cplusplus)
+#   pragma GCC diagnostic ignored "-Wc++-compat"
+#endif
 #include <X11/Xlib.h>
 #include <X11/cursorfont.h>
 #include <X11/keysym.h>
@@ -126,6 +115,7 @@
 #endif
 
 #ifdef __CYGWIN__
+#   include "tkIntXlibDecls.h"
 #   define UINT unsigned int
 #   define HWND void *
 #   define HDC void *
@@ -139,9 +129,6 @@
 #   define LPARAM void *
 #   define LRESULT void *
 
-    extern int TkPutImage(unsigned long *, int, Display *, Drawable,
-	    GC, XImage *, int, int, int, int, unsigned int, unsigned int);
-
 #else /* !__CYGWIN__ */
     /*
      * The TkPutImage macro strips off the color table information, which isn't
@@ -152,22 +139,6 @@
 		XPutImage(display, pixels, gc, image, srcx, srcy, destx, \
 		desty, width, height);
 
-    /*
-     * These macros are just wrappers for the equivalent X Region calls.
-     */
-
-#   define TkClipBox(rgn, rect) XClipBox((Region) (rgn), (rect))
-#   define TkCreateRegion() (TkRegion) XCreateRegion()
-#   define TkDestroyRegion(rgn) XDestroyRegion((Region) (rgn))
-#   define TkIntersectRegion(a, b, r) XIntersectRegion((Region) (a), \
-	(Region) (b), (Region) (r))
-#   define TkRectInRegion(r, x, y, w, h) XRectInRegion((Region) (r), (x), (y), (w), (h))
-#   define TkSetRegion(d, gc, rgn) XSetRegion((d), (gc), (Region) (rgn))
-#   define TkSubtractRegion(a, b, r) XSubtractRegion((Region) (a), \
-	(Region) (b), (Region) (r))
-#   define TkUnionRectWithRegion(rect, src, ret) XUnionRectWithRegion((rect), \
-	(Region) (src), (Region) (ret))
-
 #endif /* !__CYGWIN__ */
 
 /*
@@ -197,7 +168,7 @@
  * These functions do nothing under Unix, so we just eliminate calls to them.
  */
 
-#define TkpButtonSetDefaults(specPtr) {}
+#define TkpButtonSetDefaults() {}
 #define TkpDestroyButton(butPtr) {}
 #define TkSelUpdateClipboard(a,b) {}
 #ifndef __CYGWIN__
@@ -220,16 +191,7 @@
 
 #ifndef __CYGWIN__
 #define TkpPrintWindowId(buf,w) \
-	sprintf((buf), "%#08lx", (unsigned long) (w))
-#endif
-
-/*
- * The following declaration is used to get access to a private Tcl interface
- * that is needed for portability reasons.
- */
-
-#ifndef _TCLINT
-#include <tclInt.h>
+	sprintf((buf), "0x%08lx", (unsigned long) (w))
 #endif
 
 #endif /* _UNIXPORT */
